<?php
/**
 * @version    $Id$
 * @package    Joomla.Site
 * @subpackage  mod_languages
 * @copyright  Copyright (C) 2005 - 2011 Open Source Matters, Inc. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

// no direct access
defined('_JEXEC') or die;

jimport('joomla.language.helper');
jimport('joomla.utilities.utility');

abstract class modJdLanguageHelper {
  /**
   * Creates a link to the Active page in the given language if exists, if not redirect to the Startpage
   * @static
   * @param $lang Object the Language Object returned by jdiction::getLanguages
   * @return array With Link and Title
   */
  public static function getLink($lang) {

    $config  = JFactory::getConfig();
    $sef     = $config->get('sef');
    $rewrite = $config->get('sef_rewrite');
    $suffix  = $config->get('sef_suffix');
    $app     = JFactory::getApplication();
    $uri     = JURI::getInstance();
    $input   = $app->input;
    $menu    = $app->getMenu();
    $active  = $menu->getActive();
    $link    = false;

    $languages = $languages = JLanguageHelper::getLanguages('lang_code');
    $currentLang  = $languages[JFactory::getLanguage()->getTag()];

    // Load associations @todo regular compare with joomla original function
    $assoc = JLanguageAssociations::isEnabled();

    if ($assoc) {
      static $associations = null;
      if ($active && is_null($associations)) {
        $associations = MenusHelper::getAssociations($active->id);
      }
      // load component associations
      $option = $app->input->get('option');
      $eName  = JString::ucfirst(JString::str_ireplace('com_', '', $option));
      $cName  = JString::ucfirst($eName . 'HelperAssociation');
      JLoader::register($cName, JPath::clean(JPATH_COMPONENT_SITE . '/helpers/association.php'));

      if (class_exists($cName) && is_callable(array(
          $cName,
          'getAssociations'
        ))
      ) {
        $cassociations = call_user_func(array(
          $cName,
          'getAssociations'
        ));
      }
    }

    if (isset($cassociations[$lang->lang_code])) {
      $link = JRoute::_($cassociations[$lang->lang_code] . '&lang=' . $lang->sef);
    } elseif (isset($associations[$lang->lang_code]) && $menu->getItem($associations[$lang->lang_code])) {
      $itemid = $associations[$lang->lang_code];
      if ($app->getCfg('sef') == '1') {
        $link = JRoute::_('index.php?lang=' . $lang->sef . '&Itemid=' . $itemid);
      } else {
        $link = 'index.php?lang=' . $lang->sef . '&amp;Itemid=' . $itemid;
      }
    }

    if ($link) {
      // at this time we don't have a translated title because the HelperAssociation API does not return a title
      return array(
        $link,
        ''
      );
    }


    $db = JFactory::getDbo();

    //replace current language with target language
    $orig_translate = $db->setTranslate(true);
    $orig_lang      = $db->setLanguage($lang->lang_code);

    $menuitem = false;
    $q    = $db->getQuery(true);
    if ($active) {
      $q->select('id, home, title, menutype');
      $q->from('#__menu');
      $q->where('id=' . $db->q($active->id));
      $q->where('language in (' . $db->q($lang->lang_code) . ', ' . $db->q('*') . ')');
      $db->setQuery($q);
      $menuitem = $db->loadObject();

    }

    if ($menuitem) {
      $q->clear();
      $q->select('p.alias');
      $q->from('#__menu AS p , #__menu AS n');
      $q->where('n.lft BETWEEN p.lft AND p.rgt');
      $q->where('n.id = ' . $db->q($active->id));
      $q->where('p.alias != ' . $db->q('root'));
      $q->where('p.published > 0');
      $q->where('p.menutype = ' . $db->q($active->menutype));
      $q->where('n.menutype = ' . $db->q($active->menutype));
      $q->order('p.lft');
      $db->setQuery($q);
      $segments       = $db->loadColumn();
      $menuitem->path = trim(implode('/', $segments), ' /\\');
    }

    //restore current language
    $db->setTranslate($orig_translate);
    $db->setLanguage($orig_lang);

    $query     = array();
    $query_sef = array();

    $urlquery = $uri->getQuery(true);

    foreach ($urlquery as $k => $v) {
      switch ($k) {
        case 'option':
        case 'view':
        case 'id':
        case 'cid':
        case 'layout':
          $query[] = $k . '=' . $v;
          if (!isset($active->query[$k]) || $active->query[$k] != $v) {
            $query_sef[] = $k . '=' . $v;
          }
          break;
        case 'Itemid': // maybe different
        case 'lang': // will be changed later
          // we don't need the lang attribute
          break;
        default:
          $query[]     = $k . '=' . $v;
          $query_sef[] = $k . '=' . $v;
          break;
      }
    }

    $path = rawurldecode($uri->toString($parts = array('path')));
    $base = $uri->base(true);
    $link = $base;
    //check if we use sef_rewrite
    if (!$rewrite) {
      $link .= '/index.php';
    }

    if ($sef) {
      $link .= '/' . $lang->sef;
    } else {
      $link .= '?lang=' . $lang->sef;
    }

    $option = $app->input->get('option');
    $eName  = JString::ucfirst(JString::str_ireplace('com_', '', $option));
    $cName  = JString::ucfirst($eName . 'HelperjDiction');
    JLoader::register($cName, JPath::clean(JPATH_COMPONENT_SITE . '/helpers/jdiction.php'));
    if (class_exists($cName)) {
      $helper                 = new $cName;
      $helper->targetLanguage = $lang;
    } else {
      $helper = false;
    }

    if ($menuitem) {
      if ($sef) {
        $link .= '/';

        if (!$menuitem->home) { // startpage doesn't need a link path
          $link .= $menuitem->path;
        }

        // build the current basepath
        $base = '/'.$currentLang->sef . '/' .  $active->route;

        // get path without language prefix
        $path = JString::substr($path, JString::strlen($base), $suffix ? -5 : false);

        // load route from component helper if it exists
        if ($helper) {
          $path = $helper->getRoute($path, $input, $query, $menuitem, $active);
        }

        if ($path) {
          $link .= $path;
        }

        if ($suffix) {
          $link .= '.html';
        }

        // Append variables that are not set in the menu item
        if (!empty($query_sef)) {
          $link .= '?' . implode('&', $query_sef);
        }


      } else {
        if (!empty($query)) {
          $link .= '&' . implode('&', $query);
        }
        $link .= '&Itemid=' . $menuitem->id;
      }
    } else {
      if ($sef) {
        if (!$rewrite) {
          $base .= '/index.php';
        }
        $path = substr($path, strlen($base));

        // remove language ?1/?2en/?3more
        list(, , $path) = explode('/', $path, 3);

        if ($helper) {
          $path = $helper->getRoute($path, $input, $query, $menuitem, $active);
        }

        $link .= '/' . $path;
        if (!empty($query)) {
          $link .= '?' . implode('&', $query);
        }
      } else {
        if (!empty($query)) {
          $link .= '&' . implode('&', $query);
        }
      }

    }

    if ($menuitem) {
      $result = array(
        $link,
        $menuitem->title
      );
    } else {
      $result = array(
        $link,
        ''
      );
    }
    return $result;
  }
}
