<?php
/**
 * jDiction plugin
 *
 * @package jDiction
 * @link http://joomla.itronic.at
 * @copyright	Copyright (C) 2011 ITronic Harald Leithner. All rights reserved.
 * @license GNU General Public License v3
 *
 * This file is part of jDiction.
 *
 * jDiction is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * jDiction is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with jDiction.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
defined('_JEXEC') or die;

jimport('joomla.event.plugin');

//load jdiction framework
jimport('jdiction.jdiction');

class plgSystemjDiction extends JPlugin {

  /**
   * Construction function loading the plugin and create Logger Object if activated.
   *
   * @param object $subject
   * @param array $config
   */
  public function __construct(& $subject, $config = array()) {
		parent::__construct($subject, $config);

    $extension = JComponentHelper::getComponent('com_jdiction');
    if ($extension->params->get('debug', false)) {
      JLog::addLogger(array( 'text_file' => 'jdiction.log' ), JLog::ALL, array('jdiction'));
    }

  }
	
	/**
	 * replace database object
	 * @return void
	 */
	public function onAfterInitialise() {

		//load language file
		$lang = JFactory::getLanguage();
		$lang->load('lib_jdiction', JPATH_LIBRARIES.'/jdiction');

		$app = JFactory::getApplication();

		$jd = jDiction::getInstance();
		$jd->initialise();

    if ($app->isSite()) {
      $router = $app->getRouter();

      // attach build rules for translation on SEF
      $router->attachBuildRule(array($this, 'buildRule'), JRouter::PROCESS_AFTER);

      // attach build rules for translation on SEF
      $router->attachParseRule(array($this, 'parseRule'), JRouter::PROCESS_DURING);
    }
	}

  /**
   * Try to translates urls
   * @param $router
   * @param $uri
   */

  public function buildRule(&$router, &$uri) {
//    print_r('buildRule');
    /*
    $res =array();
    if ($uri->getVar('Itemid') == 437) {
      print_r($uri);
      jDictionMenu::loadLanguage();
      $uri->setVar('id', '38:andere-seite');
      $res = array('id'=>'38:andere-seite');
    }*/

    return array();
  }

  /**
   * Decodes translated sef urls
   * @param $router
   * @param $uri
   */
  public function parseRule(&$router, &$uri) {

    static $done = false;
    if (!$done) {
			$done = true;

			$jd = jDiction::getInstance();
			if (!$jd->getStatus()) {
				return array();
			}
			$db = JFactory::getDbo();
			$lang = JFactory::getLanguage();
			$db->setLanguage($lang->getTag());

			if ($lang->getTag() == JComponentHelper::getParams('com_languages')->get('site', 'en-GB')) {
				$db->setTranslate(false);
			} else {
				$db->setTranslate(true);
			}

      // Workaround for Joomla > 3.7.0, we need to set the correct language for the cache handler because the menu get already cached with the
      // language defined in JApplicationSite::initialiseApp(), but this language is the wrong if we change the language because language detection
      // is done in the Joomla system plugin languagefilter. The solution is to load the com_menus cache and set the correct language and reload
      // the menu, this is already done for Jooma 3.4.0, it seams that the cache works since 3.7.0
			$cache = JFactory::getCache('com_menus', 'callback');
      $cache->options['language'] = $lang->getTag();

      //Workaround for Joomla 3.4.0 bug where #__menu get queries before Joomla knows it language and is ready
			JFactory::getApplication()->getMenu()->__construct();

			//rewrite Menu route with translated alias
      $app = JFactory::getApplication();
      $menu = $app->getMenu()->getMenu();

      foreach($menu as &$item) {
        $item->route = '';
        if ($item->level > 1) {
          if (array_key_exists($item->parent_id, $menu)) {
            $item->route = $menu[$item->parent_id]->route.'/';
          }
        }
        $item->route .= $item->alias;
      }

			// load jdiction helper from component
			$option = $app->input->get('option', 'CMD');
			$eName = JString::ucfirst(JString::str_ireplace('com_', '', $option));
			$cName = JString::ucfirst($eName.'HelperjDiction');
			JLoader::register($cName, JPath::clean(JPATH_SITE . '/components/'.$option.'/helpers/jdiction.php'));
			if (class_exists($cName)) {
				$helper = new $cName;
				$helper->targetLanguage = $jd->getCurrentLanguage();
				$helper->originalLanguage = $jd->getDefaultLanguage();
				$helper->originalTranslateStatus = $db->getTranslate();
				$helper->parseRoute($router, $uri);
			}
		}

    return array();
  }
  /**
   * run after Route
   */
  public function onAfterRoute() {
    if (JFactory::getApplication()->isAdmin()) {
      $jd = jDiction::getInstance();
      $input = JFactory::getApplication()->input;
      $option = $input->get('option', null, 'cmd');
      $view = $input->get('view', null, 'cmd');
      $task = $input->get('task', null, 'cmd');
      $controller = $input->get('controller', null, 'cmd');
      if (!is_null($view) || is_null($task)) {
        if (is_null($view)) {
          $view = 'default';
        }
        $view = $jd->getView($option, $view);
      } elseif (!is_null($task)) {
        $view = $jd->getViewByTask($option, $task, $controller);
      }

      if ($view && !$view->native) {
        // Intercept the grid.id HTML Field to insert translation status
        JHtml::register('Grid.id', array($this, 'gridIdHook'));
        // Add the Toolbar in edit layout
        $jd->addToolbar();
      }
    }
  }


  /**
   * jDiction after save content method
   * Article is passed by reference, but after the save, so no changes will be saved.
   * Method is called right after the content is saved
   *
   * @param	string		$context The context of the content passed to the plugin (added in 1.6)
   * @param	object		$table A JTableContent object
   * @param	bool		$isNew If the content has just been created
   * @since	2.5
   */
  public function onContentAfterSave($context, $table, $isNew) {

    $jd = jDiction::getInstance();
    $input = JFactory::getApplication()->input;

    list($ext, $view) = explode('.', $context);

    // FOF returns always the list view
    $view = $jd->getView($ext, $input->get('view', 'default', 'cmd'));

    if (!$view) {
      return true;
    }
    $view = $view->name;

    $component = $jd->getComponent($ext);

    if (!$component) {
      return true;
    }

    $jdtable = $jd->getTableByView($ext, $view);

    if (!array_key_exists($view, $component)) {
      return true;
    }


    $db = JFactory::getDbo();
    $query = $db->getQuery(true);

    // copy variant if we save2copy
    if ($input->get('task') == 'save2copy') {

      $id = $input->getInt($jdtable->key);

      $query->select('*')->from('#__jd_store');
      $query->where($db->qn('idReference').'='.$db->q($id));
      $query->where($db->qn('referenceTable').'='.$db->q($jdtable->name));
      $query->where($db->qn('referenceOption').'='.$db->q($jdtable->component));
      $query->where($db->qn('referenceView').'='.$db->q($view));

      $db->setQuery($query);

      $translations = $db->loadObjectList();

      if ($translations) {
        foreach($translations as $translation) {
          $translation->idJdStore = null;
          $translation->idReference = $table->{$jdtable->key};
          $translation->modified = JFactory::getDate()->toSql();

          $db->insertObject('#__jd_store', $translation, 'idJdStore');
        }
      }

    }
    if ($isNew) {
      return true;
    }

    $sourcehash = $jd->getTranslationHashByView($ext, $view, $table);

    $query->clear();
    $query->update('#__jd_store');
    $query->set($db->qn('state').'='.$db->q(2));
    $query->where($db->qn('state').'='.$db->q(1));
    $query->where($db->qn('idReference').'='.$db->q($table->{$jdtable->key}));
    $query->where($db->qn('referenceTable').'='.$db->q($jdtable->name));
    $query->where($db->qn('referenceOption').'='.$db->q($jdtable->component));
    $query->where($db->qn('referenceView').'='.$db->q($view));
    $query->where($db->qn('sourcehash').'!='.$db->q($sourcehash));

    $db->setQuery($query);
    return $db->execute();
  }

  /**
   * jDiction after delete content method
   * Deletes the translation upon main item deletion
   *
   * @param	string		$context The context of the content passed to the plugin (added in 1.6)
   * @param	object		$table A JTableContent object
   * @since	2.5
   */
  public function onContentAfterDelete($context, $table) {

    $jd = jDiction::getInstance();

    list($ext, $view) = explode('.',$context);
    // FOF returns always the list view
    $view = $jd->getView($ext, JFactory::getApplication()->input->get('view', 'default', 'cmd'));
    if ($view) {
      $view = $view->name;
    }
    $component = $jd->getComponent($ext);

    if (!$component) {
      return true;
    }

    $jdtable = $jd->getTableByView($ext, $view);
    if (!array_key_exists($view, $component)) {
      return true;
    }

    $key = $jdtable->key;
    $id = $table->$key;

    $db = JFactory::getDbo();
    $query = $db->getQuery(true);

    $query->delete($db->quoteName('#__jd_store'));
    $query->where($db->qn('idReference').'='.$db->q($id));
    $query->where($db->qn('referenceTable').'='.$db->q($jdtable->name));

    $db->setQuery($query);

    if (!$db->execute()) {
      $table->setError($db->getErrorMsg());
      return false;
    }
    return true;
  }

  public function gridIdHook() {
    //force loading of JHtmlGrid
    if (!class_exists('JHtmlGrid')) {
      @include_once(JPATH_LIBRARIES.'/joomla/html/html/grid.php');
    }
		$row = func_get_arg(0);
		$id = func_get_arg(1);
		$vars = func_get_args();
		$res = call_user_func_array('JHtmlGrid::id', $vars);
		$jd = jdiction::getInstance();
    $ext = JFactory::getApplication()->input->get('option', '', 'cmd');
    $view = $jd->getView($ext, JFactory::getApplication()->input->get('view', 'default', 'cmd'));
    if (!$view) {
      return $res;
    }
    $table = $jd->getTableByView($ext, $view->name);
    $layout = $view->layout;

    JHtml::_('behavior.framework');
    //Load interface translation
		JText::script('LIB_JDICTION_TRANSLATION');
    JText::script('JSTATUS');
    JText::script('JGLOBAL_TITLE');
    $doc = JFactory::getDocument();
    $doc->addScript(JUri::root(false).'/administrator/components/com_jdiction/assets/jdiction.js');

    $result = array();

    $languages = $jd->getLanguages();
    $result['status'] = array();
    foreach($languages as $language) {
		  $result['status'][$language->image] = $jd->getTranslationStatus($table->name, $id, $language->lang_id);
    }

		$result['link'] = $jd->getTranslationLink($ext, $view->name, $layout, $id, false);
		
		// create array
    if ($row == 0) {
			$table = new stdClass;
			$table->tableselector = $view->listview->tableselector;
			if (!empty($view->listview->columnselector)) {
				$table->columnselector = $view->listview->columnselector;
			}
			$first = 'var jDictionTable = '.json_encode($table).', jdiction = {}; ';
		} else {
      $first = '';
    }
    $res .= '<script>'.$first.'jdiction['.$row.']='.json_encode($result).';</script>';

		return $res;
	
	}

  /**
   * We need to prepare custom fields per plugin because #__fields_values doesn't have a primary key
   *
   * @param $context
   * @param $item
   * @param $field
   */
  public function onCustomFieldsBeforePrepareField($context, $item, $field) {

    // We only work in frontend
    if (!JFactory::getApplication()->isClient('site')) {
      return;
    }

    $jd = jDiction::getInstance();

    list($component, $view) = explode('.', $context, 2);

    $table = $jd->getTableByView($component, $view);
    $translations = jDiction::getInstance()->getRawTranslationsByView($component, $view, $item->{$table->key});

    if (empty($translations)) {
      return;
    }

    $translations = current($translations);

    if (isset($translations['com_fields'][$field->name])) {
      $field->valueUntranslated = $field->value;
      $field->rawvalueUntranslated = $field->rawvalue;
      $field->value = $translations['com_fields'][$field->name];
      $field->rawvalue = $translations['com_fields'][$field->name];
    }

  }
}
